

#!/usr/local/bin/python

import websocket
import threading
from base64 import b64encode
import logging
import time

import xml.etree.ElementTree as ET

from .websocket_event import Websocket_Event

logger = logging.getLogger(__name__)

class Websocket_Client(object):

    def __init__(self,controller,address,port,username,password,https):
        self._address = address
        self._port = port
        self.controller = controller
        self._https = https

        self._auth = b64encode((username+':'+password).encode('UTF-8')).decode("ascii")

        self._headers = [
            'Authorization: Basic %s' % self._auth, 
            "Sec-WebSocket-Protocol: ISYSUB",
            "Origin: com.universal-devices.websockets.isy"
        ]

        self._connected = False
        self.connect()
    
    @property
    def connected(self):
        return self._connected

    @connected.setter
    def connected(self,connected):
        self._connected = connected
        if self.controller:
            self.controller.websocket_connected(connected)

    def connect(self):
        self._ws = websocket.WebSocketApp(
            "ws://"+self._address+"/rest/subscribe",
            header=self._headers,
            on_open = lambda ws : self._on_open (ws),
            on_message = lambda ws,message : self._on_message (ws,message),
            on_error = lambda ws,err : self._on_error (ws,err),
            on_close = lambda ws : self._on_close (ws),
        )
        
        self._ws_thread = threading.Thread(
            target=self._ws.run_forever, args=())
            
        self._ws_thread.daemon = True
        self._ws_thread.start()
        
    def _on_open(self,ws):
        logger.info('Connected')
        self.connected = True

    def _on_message(self,ws, message):
        logger.debug('Websocket Message: {}'.format(message))
        #print('Websocket Message: {}'.format(message))

        try:
            event_node = ET.fromstring (message)        
            
            if event_node.tag == 'Event':
                event = Websocket_Event(event_node)

                if event.valid:
                    if self.controller:
                        self.controller.websocket_event(event)
        
        except Exception as ex:
            logger.error('Websocket On Message Error {}'.format(ex))

    def _on_error(self,ws, error):
        logger.error('Websocket Error: {}'.format(error))

    def _on_close(self,ws):
        logger.info('Websocket Disonnected')
        self.connected = False
        self.connect()




if __name__ == "__main__":
    websocket.enableTrace(True)

    try:
        wsc = Websocket_Client(False,"192.168.1.213",80,'admin','admin',False)

        while True:
            time.sleep(2)

    except KeyboardInterrupt:
        print("KeyboardInterrupt has been caught.")




#! /usr/bin/env python

'''

container for a collection of items

container_type = device, variable, program, scene

'''

import copy

import logging
logger = logging.getLogger(__name__)

#containter events = {'add','remove','property'}

class Item_Container (object):

    def __init__(self, controller, container_type):
        self.controller = controller
        self.container_type = container_type
        self.items_retrieved = False

        self.list = {} # indexed by device(node) address

        self.started = False

    def start(self):
        pass
       
    def send_request(self,path,query=None,timeout=None): 
        return self.controller.send_request(path,query,timeout)

    def websocket_event(self,event):
        pass

    def add(self,item,key):
        self.list [key] = item
        self.event (item,'add')

    def remove(self,key):
        item = self.list [key]
        del self.list [key]
        self.event (item,'remove')

    def get(self,key):
        if key in self.list:
            return self.list [key]
        else:
            return None

    def get_list_copy(self):
        return copy.copy(self.list)

    def property_change(self,item,property_,value): # called by the item to publish a change
        self.event(item,'property',property_,value)
    
    def event(self,item,event,*args): #publish event to controller
        #print ('item event',self.container_type,item.name,event)
        self.controller.container_event (self,item,event,args)
    
    
        




#! /usr/bin/env python

import xml.etree.ElementTree as ET

class Variable_Info(object):

    def __init__(self,node):

        self.valid = False

        try:

            self.id = node.attrib['id']
            self.type = node.attrib['type']
            self.init_value = int(node.find('init').text)
            self.value = int(node.find('val').text)
            self.time_set = node.find('ts').text

            self.valid = True

        except:
            pass

    def __repr__(self):
        return 'Variable: ID {} Value {}, Init {}, Time {}'.format(self.id,self.value,self.init_value,self.time_set)
    


#! /usr/bin/env python

''' 

Base Device

Common to all ISY items ie programs, variables, devices, scenes

status property -> ready, alert, lost


'''



class Item_Base(object):

    def __init__(self, container, name):
        self.container = container

        self.name = name

        self.properties = {'status' : 'init'} # list of properties key = property name, value = property value

        self.property_event_handlers = []

    def process_websocket_event(self,event):
        pass # classes to override

    def add_property(self, property_, value = None):
        self.properties [property_] = value

    def set_property(self, property_, value, always_publish=False): # propagates up to the container and to handlers
        if self.properties [property_] != value or always_publish:
            self.properties [property_] = value
            self.container.property_change(self,property_,value) 

            for handler in self.property_event_handlers:
                handler (property_,value)
 
    def get_property(self, property_):
        return self.properties [property_] 

    def send_request(self,path,query=None,timeout=None): 
        return self.container.send_request(path,query,timeout)

    def add_property_event_handler(self,handler):
        self.property_event_handlers.append(handler)

    def get_identifier(self):
        pass # subclasses to provide


#! /usr/bin/env python

import requests
from urllib.parse import urlencode

import logging
logger = logging.getLogger(__name__)

TIMEOUT = 3

class HTTP_Client(object):

    def __init__(self,address,port=None,username='',password='',use_https=False):

        self._address = address
        self._port = port
        self._username = username
        self._password = password

        url = (use_https and 'https://' or 'http://') + self._address
        # add port...
        self._url = url

        self.create_session()

    def create_session(self):
        self._session = requests.Session()
        self._session.auth = (self._username,self._password)

    def compile_request(self, path, query=None):
        url = self._url + '/rest' + '/' + path 
        if query is not None:
            url = url + '?' + query

        return url

    def request(self,path,query=None, timeout=TIMEOUT): #TBD better error handling and logging
        url = self.compile_request(path,query)

        try:
            logger.debug('Request: {}'.format(url,timeout=timeout))
            response = self._session.get (url)
            response.raise_for_status()
            return True,response
        
        except requests.exceptions.RequestException as err:
            logger.warn('Request Error: {}'.format(err))
            return None,err
        except requests.exceptions.HTTPError as errh:
            logger.warn('Request HTTP Error: {}'.format(errh))
            return None,errh
        except requests.exceptions.ConnectionError as errc:
            logger.warn('Request Connection Error: {}'.format(errc))
            return None,errc
        except requests.exceptions.Timeout as errt:
            logger.warn('Request Timeout Error: {}'.format(errt))
            return None, errt


#! /usr/bin/env python

# process an event from ISY web socket client

class Websocket_Event(object):

    def __init__(self,event):

        self.event = event

        self.valid = False

        try:
            node = event.find('node')
            control = event.find ('control')
            action = event.find ('action')
            event_info = event.find('eventInfo')

            self.address = node.text 
            self.control = control.text 
            self.action = action.text 
            self.event_info = event_info.text 

            self.event_info_node = event_info #store as consumers may need to use it

            self.valid = True

        except:
            pass 

    def __repr__(self):
        return 'Event: Address {}, Control {}, Action {}, Event Info {}'.format(self.address,self.control,self.action,self.event_info)
    